// ===========================================================================
//	MacPLL.cp 		1996 Mikhail Fridberg. All rights reserved.
// ===========================================================================
//

#include "MacPLL.h"
#include "MacPLLConstants.h"

#include <LGrowZone.h>
#include <LWindow.h>
#include <PP_Messages.h>
#include <PP_Resources.h>
#include <PPobClasses.h>
#include <UDrawingState.h>
#include <UMemoryMgr.h>
#include <URegistrar.h>
#include <LEditField.h>
#include <LMenu.h>
#include <LMenuBar.h>

#include <fp.h>
//#include <math.h>

// ===========================================================================
//		 Main Program
// ===========================================================================

void main(void)
{
									// Set Debugging options
	SetDebugThrow_(debugAction_Alert);
	SetDebugSignal_(debugAction_Alert);

	InitializeHeap(3);				// Initialize Memory Manager
									// Parameter is number of Master Pointer
									//   blocks to allocate
	
									// Initialize standard Toolbox managers
	UQDGlobals::InitializeToolbox(&qd);
	
	new LGrowZone(20000);			// Install a GrowZone function to catch
									//    low memory situations.
	ThrowIfMemError_ ();


	CPPMacPLLApp	theApp;			// replace this with your App type
	theApp.Run();
}


// ---------------------------------------------------------------------------
//		 CPPStarterApp 			// replace this with your App type
// ---------------------------------------------------------------------------
//	Constructor

CPPMacPLLApp::CPPMacPLLApp()
{
	// Register functions to create core PowerPlant classes
	
	RegisterAllPPClasses();
	mPLLSystem = new PLL_Params;
	ThrowIfMemError_ ();
	ThrowIfMemFail_ (mPLLSystem);
	thetaArray=new double_t[THETA_ARRAY_SIZE];
	ThrowIfMemError_ ();
	ThrowIfMemFail_ (thetaArray);
}


// ---------------------------------------------------------------------------
//		 ~CPPStarterApp			// replace this with your App type
// ---------------------------------------------------------------------------
//	Destructor
//

CPPMacPLLApp::~CPPMacPLLApp()
{
delete mPLLSystem;
delete thetaArray;
}

// ---------------------------------------------------------------------------
//		 StartUp
// ---------------------------------------------------------------------------

void
CPPMacPLLApp::StartUp()
{
	ObeyCommand(cmd_New, nil);
}

// ---------------------------------------------------------------------------
//		 ObeyCommand
// ---------------------------------------------------------------------------
//	Respond to commands

Boolean
CPPMacPLLApp::ObeyCommand(
	CommandT	inCommand,
	void		*ioParam)
{
	Boolean		cmdHandled = true;

	switch (inCommand) { 			
		case cmd_New:
			mWindow = MakeControlsWindow();
			break;
		default:
			cmdHandled = LApplication::ObeyCommand(inCommand, ioParam);
			break;
	}
	
	return cmdHandled;
}

// ---------------------------------------------------------------------------------
//		 MakeControlsWindow
// ---------------------------------------------------------------------------------
LWindow *CPPMacPLLApp::MakeControlsWindow()
{
	// Create the window.
	LWindow		*theWindow;
//	theWindow = LWindow::CreateWindow( main_Window, this );
	theWindow = LWindow::CreateWindow( main_Window, this );
	
	// Link the application (the listener) with the controls in
	// the window (the broadcasters).
	UReanimator::LinkListenerToControls(this,theWindow, main_Window);

	// Show the window.
	theWindow->Show();
	return theWindow;
}

// ---------------------------------------------------------------------------
//		 FindCommandStatus
// ---------------------------------------------------------------------------
//	This function enables menu commands.
//

void
CPPMacPLLApp::FindCommandStatus(
	CommandT	inCommand,
	Boolean		&outEnabled,
	Boolean		&outUsesMark,
	Char16		&outMark,
	Str255		outName)
{

	switch (inCommand) {
	
		// Return menu item status according to command messages.
		// Any that you don't handle will be passed to LApplication

		case cmd_New:
			outEnabled = true;		// enable the New command
		break;
		
		default:
			LApplication::FindCommandStatus(inCommand, outEnabled,
												outUsesMark, outMark, outName);
			break;
	}
}

// ---------------------------------------------------------------------------------
//		 ListenToMessage
// ---------------------------------------------------------------------------------
void
CPPMacPLLApp::ListenToMessage(
	MessageT	inMessage,
	void		*ioParam )
{
#pragma unused( ioParam )
	
	static	LPicture*		myPictures;
	static	LCaption*		theExtrCaption;
	static	LEditField*		myEditFiled;
	Int16					theDetectorType;
	Int16					theFilterType;

	// make sure that we we have correct window
	mWindow = UDesktop::FetchTopRegular();
// identify source of messages

	switch (inMessage) {
		
	//hide or activate appropriate controls:

	//Find what is selected in pop-ups and adjust appearance
		case	msg_PhaseDetectorPopUp:
		case	msg_LoopFilterPopUp:
		{
		
		//First, lets find what are all 2 parameters are:
			GetInputPopUps();

			//Depending on detector type, put appropriate picture here
			theDetectorType = mPLLSystem->DetectorType;
			theFilterType = mPLLSystem->FilterType;
			switch(theDetectorType)
				{
				case Type_1:
					myPictures = (LPicture*)mWindow->FindPaneByID(DetectorTypePicture);
					myPictures->SetPictureID(Detector_1);
					myPictures->Refresh();
					//Lock range is defined for all types phase detectors except 4
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kLockRangeCaption);
					theExtrCaption->Show();
					theExtrCaption->Refresh();
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kLockRangeUnitsCaption);
					theExtrCaption->Show();
					theExtrCaption->Refresh();
					myEditFiled = (LEditField*)mWindow->FindPaneByID(kLockRange);
					myEditFiled->Show();
					myEditFiled->Refresh();
					break;
				case Type_2:
					myPictures = (LPicture*)mWindow->FindPaneByID(DetectorTypePicture);
					myPictures->SetPictureID(Detector_2);
					myPictures->Refresh();
					//Lock range is defined for all types phase detectors except 4
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kLockRangeCaption);
					theExtrCaption->Show();
					theExtrCaption->Refresh();
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kLockRangeUnitsCaption);
					theExtrCaption->Show();
					theExtrCaption->Refresh();
					myEditFiled = (LEditField*)mWindow->FindPaneByID(kLockRange);
					myEditFiled->Show();
					myEditFiled->Refresh();
					break;
				case Type_3:
					myPictures = (LPicture*)mWindow->FindPaneByID(DetectorTypePicture);
					myPictures->SetPictureID(Detector_3);
					myPictures->Refresh();
					//Lock range is defined for all types phase detectors except 4
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kLockRangeCaption);
					theExtrCaption->Show();
					theExtrCaption->Refresh();
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kLockRangeUnitsCaption);
					theExtrCaption->Show();
					theExtrCaption->Refresh();
					myEditFiled = (LEditField*)mWindow->FindPaneByID(kLockRange);
					myEditFiled->Show();
					myEditFiled->Refresh();
					break;
				case Type_4:
					myPictures = (LPicture*)mWindow->FindPaneByID(DetectorTypePicture);
					myPictures->SetPictureID(Detector_4);
					myPictures->Refresh();
					//Lock range is not defined for type 4 phase detectors
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kLockRangeCaption);
					theExtrCaption->Hide();
					theExtrCaption->Refresh();
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kLockRangeUnitsCaption);
					theExtrCaption->Hide();
					theExtrCaption->Refresh();
					myEditFiled = (LEditField*)mWindow->FindPaneByID(kLockRange);
					myEditFiled->Hide();
					myEditFiled->Refresh();
					break;		
				}
	
			//Depending on filter type, put appropriate picture here
			switch(theFilterType)
				{
				case Type_1:
					myPictures = (LPicture*)mWindow->FindPaneByID(FilterTypePicture);
					myPictures->SetPictureID(Filter_1);
					myPictures->Refresh();
					//Hide anything associated with R2, since we dont have it for
					//this type of filter
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kR2Caption);
					theExtrCaption->Hide();
					theExtrCaption->Refresh();
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kR2UnitsCaption);
					theExtrCaption->Hide();
					theExtrCaption->Refresh();
					myEditFiled = (LEditField*)mWindow->FindPaneByID(kR2);
					myEditFiled->Hide();
					myEditFiled->Refresh();
					break;
				case Type_2:
					myPictures = (LPicture*)mWindow->FindPaneByID(FilterTypePicture);
					myPictures->SetPictureID(Filter_2);
					myPictures->Refresh();
					//Show R2 exist in lead-lag filter
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kR2Caption);
					theExtrCaption->Show();
					theExtrCaption->Refresh();
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kR2UnitsCaption);
					theExtrCaption->Show();
					theExtrCaption->Refresh();
					myEditFiled = (LEditField*)mWindow->FindPaneByID(kR2);
					myEditFiled->Show();
					myEditFiled->Refresh();
					break;
				case Type_3:
					myPictures = (LPicture*)mWindow->FindPaneByID(FilterTypePicture);
					myPictures->SetPictureID(Filter_3);
					myPictures->Refresh();
					//Show R2 exist in charge pump filter
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kR2Caption);
					theExtrCaption->Show();
					theExtrCaption->Refresh();
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kR2UnitsCaption);
					theExtrCaption->Show();
					theExtrCaption->Refresh();
					myEditFiled = (LEditField*)mWindow->FindPaneByID(kR2);
					myEditFiled->Show();
					myEditFiled->Refresh();
					break;
				case Type_4:
					myPictures = (LPicture*)mWindow->FindPaneByID(FilterTypePicture);
					myPictures->SetPictureID(Filter_4);
					myPictures->Refresh();
					//Hide anything associated with R2, since we dont have it for
					//this type of filter
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kR2Caption);
					theExtrCaption->Hide();
					theExtrCaption->Refresh();
					theExtrCaption = (LCaption*)mWindow->FindPaneByID(kR2UnitsCaption);
					theExtrCaption->Hide();
					theExtrCaption->Refresh();
					myEditFiled = (LEditField*)mWindow->FindPaneByID(kR2);
					myEditFiled->Hide();
					myEditFiled->Refresh();
					break;		
				}

		}
		break;
		case	msg_Calculate:
		{
		//If Calculate button pressed, first check if all data present and if its valid
			if (CheckInputDataIntegrity() == DATA_INVALID) break;

		//Then calculate filter and other parameters of the PLL

			//Set mouse pointer to watch
			CursHandle	hCurs = GetCursor(watchCursor);
			Cursor		waitCursor = **hCurs;
			SetCursor(&waitCursor);
			
			GetInputPopUps();
			CalculateLoopFilter();
		//And display the results
			CheckOutputDataIntegrity();
			ShowTheResults();
			// return mouse pointer back to arrow
			SetCursor(&qd.arrow);
			SysBeep(10);
		}
		break;
	}
	
}
//----------------------------------------------------------------------------------
//		FindDetectorType()
// ---------------------------------------------------------------------------------
Int16	CPPMacPLLApp::FindDetectorType(Str255 mDetectorType)
{
//Create new LStr255 object so I can use == for comparison
	LStr255	mLSr255 = mDetectorType;

//Now get the values of from pop-up

		if (mLSr255 == "\pType 1") {
			return Type_1;
		}
		if (mLSr255 == "\pType 2") {
			return Type_2;
		}
		if (mLSr255 == "\pType 3") {
			return Type_3;
		}
		if (mLSr255 == "\pType 4") {
			return Type_4;
		}
//If there are problem for some reasons, return -1
	return PROBLEM;
	
}

//----------------------------------------------------------------------------------
//		FindLoopFilterType()
// ---------------------------------------------------------------------------------
Int16	CPPMacPLLApp::FindLoopFilterType( Str255 mFilterType)
{
//Create new LStr255 object so I can use == for comparison
	LStr255	mLSr255 = mFilterType;

//Now get the values of from pop-up

		if (mLSr255 == "\pType 1") {
			return Type_1;
		}
		if (mLSr255 == "\pType 2") {
			return Type_2;
		}
		if (mLSr255 == "\pType 3") {
			return Type_3;
		}
		if (mLSr255 == "\pType 4") {
			return Type_4;
		}
//If there are problem for some reasons, return -1
	return PROBLEM;
	
}

//----------------------------------------------------------------------------------
//		CheckInputDataIntegrity()
// ---------------------------------------------------------------------------------
Int16	CPPMacPLLApp::CheckInputDataIntegrity(void)
{
LEditField*		myEditFiled;
TEHandle		myText;

//First, read data and fill in structure that holds it.
//Since integer entries, such as NMin, NMax, etc are already checked
//by appropriate flags in LEditField in Constructor, dont check for it.
//Just make sure that data is whithin range, and refuse to proceed if it isnt.
//Also check if it is whithin range, but questionable, and give warning.


	myEditFiled = (LEditField*)mWindow->FindPaneByID(kReferenseFreq);
	myText = myEditFiled->GetMacTEH();
	mPLLSystem->RefGeneratorFrequency = LStr255(TEGetText(myText));
	mPLLSystem->RefGeneratorFrequency *= (1.0e6 * pi * 2) ; //input in MHz, convert to radians.
	
	myEditFiled = (LEditField*)mWindow->FindPaneByID(kReferenceDivider);
	myText = myEditFiled->GetMacTEH();
	mPLLSystem->ReferenceDivider = LStr255(TEGetText(myText));

	myEditFiled = (LEditField*)mWindow->FindPaneByID(kPhaseDetectorGain);
	myText = myEditFiled->GetMacTEH();
	mPLLSystem->PhaseDetectorGain = LStr255(TEGetText(myText));

	myEditFiled = (LEditField*)mWindow->FindPaneByID(kVCOGain);
	myText = myEditFiled->GetMacTEH();
	mPLLSystem->VCOGain = LStr255(TEGetText(myText));
	mPLLSystem->VCOGain *= (1.0e6 * pi * 2); //input in Mhz/V, we convert it in radians/v 

	myEditFiled = (LEditField*)mWindow->FindPaneByID(kOUTFreqMin);
	myText = myEditFiled->GetMacTEH();
	mPLLSystem->OUTFreqMin = LStr255(TEGetText(myText));
	mPLLSystem->OUTFreqMin *= (1.0e6 * pi * 2); //input in Mhz, we convert it in radians/v 

	myEditFiled = (LEditField*)mWindow->FindPaneByID(kOUTFreqMax);
	myText = myEditFiled->GetMacTEH();
	mPLLSystem->OUTFreqMax = LStr255(TEGetText(myText));
	mPLLSystem->OUTFreqMax *= (1.0e6 * pi * 2); //input in Mhz, we convert it in radians/v 

	myEditFiled = (LEditField*)mWindow->FindPaneByID(kLockUpTime);
	myText = myEditFiled->GetMacTEH();
	mPLLSystem->LockUpTime = LStr255(TEGetText(myText));
	mPLLSystem->LockUpTime *=0.001; //inut in milliseconds, convert to seconds

	myEditFiled = (LEditField*)mWindow->FindPaneByID(kZeta);
	myText = myEditFiled->GetMacTEH();
	mPLLSystem->ZetaMin = LStr255(TEGetText(myText));

	myEditFiled = (LEditField*)mWindow->FindPaneByID(kOvershoot);
	myText = myEditFiled->GetMacTEH();
	mPLLSystem->Overshoot = LStr255(TEGetText(myText));
	mPLLSystem->Overshoot /= 100.0; //input in percents, convert to times

	myEditFiled = (LEditField*)mWindow->FindPaneByID(kR1nput);
	myText = myEditFiled->GetMacTEH();
	mPLLSystem->R1 = LStr255(TEGetText(myText));
	mPLLSystem->R1 *= 1000.0; //input in kOhms, convert to ohms.
//Next, make sure that it is whithin limits
	
	if ((mPLLSystem->R1 <= 0.0) || isnan(mPLLSystem->R1)) {
		HideShowErrors(kR1nputError, SetError);
		return DATA_INVALID;
	}
	if (mPLLSystem->Overshoot <= 0.0 || mPLLSystem->Overshoot >= 1.0 || isnan(mPLLSystem->Overshoot)) {
		HideShowErrors(kOvershootError, SetError);
		return DATA_INVALID;
	}
	if (mPLLSystem->ZetaMin <= 0.0 || mPLLSystem->ZetaMin >= 10.0 || isnan(mPLLSystem->ZetaMin)) {
		HideShowErrors(kZetaError, SetError);
		return DATA_INVALID;
	}
	if (isnan(mPLLSystem->LockUpTime) || (mPLLSystem->LockUpTime <= 0.0)){ 
		HideShowErrors(kLockUpTimeError, SetError);
		return DATA_INVALID;
	}
	if (isnan(mPLLSystem->VCOGain) || (mPLLSystem->VCOGain <= 0.0)) {
		HideShowErrors(kVCOGainError, SetError);
		return DATA_INVALID;
	}
	if (isnan(mPLLSystem->RefGeneratorFrequency) || (mPLLSystem->RefGeneratorFrequency <= 0.0)) {
		HideShowErrors(kReferenseFreqError, SetError);
		return DATA_INVALID;
	}
	if (isnan(mPLLSystem->PhaseDetectorGain) || (mPLLSystem->PhaseDetectorGain <= 0.0)) {
		HideShowErrors(kPhaseDetectorGainError, SetError);
		return DATA_INVALID;
	}
	if (isnan(mPLLSystem->ReferenceDivider) || mPLLSystem->ReferenceDivider <= 0) {
		HideShowErrors(kReferenceDividerError, SetError);
		return DATA_INVALID;
	}
	if (isnan(mPLLSystem->OUTFreqMin) || mPLLSystem->OUTFreqMin <= 0.0) {
		HideShowErrors(kOUTFreqMinError, SetError);
		return DATA_INVALID;
	}
	if (isnan(mPLLSystem->OUTFreqMax) || mPLLSystem->OUTFreqMax <= 0.0) {
		HideShowErrors(kOUTFreqMaxError, SetError);
		return DATA_INVALID;
	}
	if (mPLLSystem->OUTFreqMax < mPLLSystem->OUTFreqMin) {
		HideShowErrors(kOUTFreqMaxError, SetError);
		HideShowErrors(kOUTFreqMinError, SetError);
		return DATA_INVALID;
	}
	if (mPLLSystem->OUTFreqMin < mPLLSystem->RefGeneratorFrequency/mPLLSystem->ReferenceDivider) {
		HideShowErrors(kOUTFreqMaxError, SetError);
		HideShowErrors(kOUTFreqMinError, SetError);
		HideShowErrors(kReferenseFreqError, SetError);
		HideShowErrors(kReferenceDividerError, SetError);
		return DATA_INVALID;
	}
	
	
	HideShowErrors(kPhaseDetectorGainError, ClearError);
	HideShowErrors(kReferenseFreqError, ClearError);
	HideShowErrors(kReferenceDividerError, ClearError);
	HideShowErrors(kVCOGainError, ClearError);
	HideShowErrors(kOUTFreqMinError, ClearError);
	HideShowErrors(kOUTFreqMaxError, ClearError);
	HideShowErrors(kLockUpTimeError, ClearError);
	HideShowErrors(kZetaError, ClearError);
	HideShowErrors(kR1nputError, ClearError);
	HideShowErrors(kOvershootError, ClearError);

	
return NO_ERRORS;
}

//----------------------------------------------------------------------------------
//		CheckOutputDataIntegrity()
// ---------------------------------------------------------------------------------
Int16	CPPMacPLLApp::CheckOutputDataIntegrity(void)
{
	if (isnan(mPLLSystem->R1)) HideShowErrors(kR1Error, SetError);
		else HideShowErrors(kR1Error, ClearError);
	if (isnan(mPLLSystem->R2)) HideShowErrors(kR2Error, SetError);
		else HideShowErrors(kR2Error, ClearError);
	if (isnan(mPLLSystem->C)) HideShowErrors(kCError, SetError);
		else HideShowErrors(kCError, ClearError);
	if (isnan(mPLLSystem->Cc)) HideShowErrors(kCcError, SetError);
		else HideShowErrors(kCcError, ClearError);
	if (isnan(mPLLSystem->OmegaN)) HideShowErrors(kOmegaNError, SetError);
		else HideShowErrors(kOmegaNError, ClearError);
	if (isnan(mPLLSystem->ZetaMax)) HideShowErrors(kZetaMaxError, SetError);
		else HideShowErrors(kZetaMaxError, ClearError);
	if (isnan(mPLLSystem->NMin)) HideShowErrors(kDividerNMinError, SetError);
		else HideShowErrors(kDividerNMinError, ClearError);
	if (isnan(mPLLSystem->NMax)) HideShowErrors(kDividerNMaxError, SetError);
		else HideShowErrors(kDividerNMaxError, ClearError);
	if (isnan(mPLLSystem->VCORangeMin)) HideShowErrors(kVCORangeMinError, SetError);
		else HideShowErrors(kVCORangeMinError, ClearError);
	if (isnan(mPLLSystem->VCORangeMax)) HideShowErrors(kVCORangeMaxError, SetError);
		else HideShowErrors(kVCORangeMaxError, ClearError);
	if (isnan(mPLLSystem->Bandwidth3db)) HideShowErrors(kBandwidth3dbError, SetError);
		else HideShowErrors(kBandwidth3dbError, ClearError);
	if (isnan(mPLLSystem->MaxOvershoot)) HideShowErrors(kMaxOvershootError, SetError);
		else HideShowErrors(kMaxOvershootError, ClearError);
	if (isnan(mPLLSystem->LockRange)) HideShowErrors(kLockRangeError, SetError);
		else HideShowErrors(kLockRangeError, ClearError);
	if (isnan(mPLLSystem->PullInRange)) HideShowErrors(kPullInRangeError, SetError);
		else HideShowErrors(kPullInRangeError, ClearError);
	if (isnan(mPLLSystem->HoldRange)) HideShowErrors(kHoldRangeError, SetError);
		else HideShowErrors(kHoldRangeError, ClearError);
	if (isnan(mPLLSystem->PullOutRange)) HideShowErrors(kPullOutRangeError, SetError);
		else HideShowErrors(kPullOutRangeError, ClearError);

return NO_ERRORS;
}
//----------------------------------------------------------------------------------
//		CalculateLoopFilter()
// ---------------------------------------------------------------------------------
Int16	CPPMacPLLApp::CalculateLoopFilter(void)
{
double_t	NMax, NMin, VCORangeMin, VCORangeMax, OUTFreqMin, OUTFreqMax;
double_t	RefGeneratorFrequency, ReferenceDivider, VCOGain, LockUpTime;
double_t	OmegaN, C, PhaseDetectorGain, R1, R2, ZetaMax, ZetaMin, Cc, Bandwidth3db;
double_t	MaxOvershoot, OmegaNMin, OmegaNMax, LockRange, PullInRange;
double_t	HoldRange, PullOutRange, Zeta;

//copy all data from a structure into local variables.
	OUTFreqMax = mPLLSystem->OUTFreqMax;
	OUTFreqMin = mPLLSystem->OUTFreqMin;
	RefGeneratorFrequency = mPLLSystem->RefGeneratorFrequency;
	ReferenceDivider = mPLLSystem->ReferenceDivider;
	LockUpTime = mPLLSystem->LockUpTime;
	PhaseDetectorGain = mPLLSystem->PhaseDetectorGain;
	VCOGain = mPLLSystem->VCOGain; 
	ZetaMin = mPLLSystem->ZetaMin;
	R1 = mPLLSystem->R1;

//Then calculate the params
	NMax = ceil(OUTFreqMax/(RefGeneratorFrequency/ReferenceDivider));
	NMin = floor(OUTFreqMin/(RefGeneratorFrequency/ReferenceDivider));
	ZetaMax = ZetaMin * sqrt(NMax/NMin);
	mPLLSystem->ZetaMax = ZetaMax;
	Zeta = (ZetaMin+ZetaMax)/2;
	OmegaNMax = CalculateTransientResponce(ZetaMax)/LockUpTime;
	OmegaNMin = CalculateTransientResponce(ZetaMin)/LockUpTime;
	OmegaN = CalculateTransientResponce(Zeta)/LockUpTime;
Int16	NAvg = (NMax+NMin)/2;
	C = (PhaseDetectorGain * VCOGain)/(NAvg * pow(OmegaN,2) * R1);
	R2 = (2 * Zeta)/(OmegaN * C);
	Cc = 0.8/(R1 * OmegaN);
	Bandwidth3db = OmegaN * sqrt(1 + 2 * pow(Zeta,2) + sqrt(2 + 4 * pow(Zeta,2) +
					4 * pow(Zeta,4)));
	MaxOvershoot = FindMaxOvershoot();
	VCORangeMin = OUTFreqMin * (1 -  MaxOvershoot);
	VCORangeMax = OUTFreqMax * (1 +  MaxOvershoot);


// If we are using detector of type 1 (linear analog (double)balanced mixer/modulator/multiplier), the loop dynamic will be
//slightly different then for detectors type 2 and 3, and more different then detector type 4. Strictly speaking, such linear,
//or analog PLL will not have dividers in it, and if used as frequency multiplier will work on harmonics of reference
// signal, instead of dividing VCO signal down. While this is a valid use for fully-analog PLL, its not an optimal 
//(with few exceptions), since syntesizers can handle task of frequency multiplying much better. Analog PLLs are mostly used as
// FM and PM modulators and demodulators. In case of using PLL as modulators/demodulators, different set of parameters
//then calculated here are of interest, such as max phase error for given frequency deviation, max frequency deviation,
//max tracking speed, max modulation frequency, etc.
//Of course it is possible to use analog PD in freq. sintesizers, etc. especially if we are using square waves insted of
//sinusoidal signals for both VCO output and reference input, but even then such PDs are not used very often, since
//digital PD, especially type 4, offer better performance. In any event, calculations for PD type 1 are performed 
//assuming square waves on its inputs, and analog output.
	
	if (mPLLSystem->DetectorType == Type_1) {
		if ((mPLLSystem->FilterType == Type_1) || (mPLLSystem->FilterType == Type_4))
			LockRange = sqrt((PhaseDetectorGain * VCOGain)/(R1 * C));
		if ((mPLLSystem->FilterType == Type_2) || (mPLLSystem->FilterType == Type_3))
			LockRange = 2  * Zeta *OmegaN;
		PullInRange = (8/pi) * sqrt((Zeta*OmegaN*PhaseDetectorGain*VCOGain/NMax) - pow(OmegaN,2));
		if ((mPLLSystem->FilterType == Type_1) || (mPLLSystem->FilterType == Type_2))
			HoldRange = PhaseDetectorGain * VCOGain/NAvg;
		if ((mPLLSystem->FilterType == Type_3) || (mPLLSystem->FilterType == Type_4))
			HoldRange = INF;
		PullOutRange = 1.8 * OmegaN *(Zeta + 1);
	}

//For all other types of PD (DPLL)
	//Find hold range in radians/sec
	if ((mPLLSystem->FilterType == Type_1 || mPLLSystem->FilterType == Type_2) && (mPLLSystem->DetectorType == Type_2))
		HoldRange = pi * PhaseDetectorGain * VCOGain/(2 * NAvg);
	if ((mPLLSystem->FilterType == Type_1 || mPLLSystem->FilterType == Type_2) && (mPLLSystem->DetectorType == Type_3))
		HoldRange = pi * PhaseDetectorGain * VCOGain/NAvg;
	if ((mPLLSystem->FilterType == Type_3) || (mPLLSystem->FilterType == Type_4) || (mPLLSystem->DetectorType == Type_4))
		HoldRange = INF; //Theoretical infinity, practically limited by components

	//Find lock range in radians/sec

	if ((mPLLSystem->FilterType == Type_2 || mPLLSystem->FilterType == Type_3) && 
																	(mPLLSystem->DetectorType == Type_2))
			LockRange = pi * Zeta * OmegaN;
	if ((mPLLSystem->FilterType == Type_1 || mPLLSystem->FilterType == Type_4) && 
																	(mPLLSystem->DetectorType == Type_2))
			LockRange = pi * OmegaN/sqrt(8.0);

	if ((mPLLSystem->FilterType == Type_2 || mPLLSystem->FilterType == Type_3) && 
																	(mPLLSystem->DetectorType == Type_3))
			LockRange = 2 * pi * Zeta * OmegaN;
	if ((mPLLSystem->FilterType == Type_1 || mPLLSystem->FilterType == Type_4) && 
																	(mPLLSystem->DetectorType == Type_3))
			LockRange = pi * OmegaN/sqrt(8.0);
	
	if (mPLLSystem->DetectorType == Type_4)
			LockRange = INF;	//Lock-in process for type 4 pd is not defined, its essentially a pull-in process,
								//which has theoretical range of infinity but practically limited by components

	//Pull-in range in rad/sec
	if ((mPLLSystem->FilterType == Type_3 || mPLLSystem->FilterType == Type_4) && 
																	(mPLLSystem->DetectorType == Type_2))
			PullInRange = (pi/2) * sqrt(2 * Zeta * OmegaN * VCOGain * PhaseDetectorGain/NAvg);
	
	if ((mPLLSystem->FilterType == Type_1 || mPLLSystem->FilterType == Type_2) && 
																	(mPLLSystem->DetectorType == Type_2))
			PullInRange = (pi/2) * sqrt((2 * Zeta * OmegaN * VCOGain * PhaseDetectorGain/NAvg) - pow(OmegaN,2));
	
	if ((mPLLSystem->FilterType == Type_3 || mPLLSystem->FilterType == Type_4) && 
																	(mPLLSystem->DetectorType == Type_3))
			PullInRange = pi * sqrt(2 * Zeta * OmegaN * VCOGain * PhaseDetectorGain/NAvg);
	
	if ((mPLLSystem->FilterType == Type_1 || mPLLSystem->FilterType == Type_2) && 
																	(mPLLSystem->DetectorType == Type_3))
			PullInRange = pi * sqrt((2 * Zeta * OmegaN * VCOGain * PhaseDetectorGain/NAvg) - pow(OmegaN,2));
			
	if (mPLLSystem->DetectorType == Type_4)
			PullInRange = INF; //Theoretical infinity, practically limited by components
	
	//Pull-out range in rad/sec
	if (mPLLSystem->DetectorType == Type_2)
			PullOutRange = 1.8 * OmegaN *(Zeta + 1);
	
	//zeta = 1.0 will turn all of 4 following equations into pi* omegan * e (e = base of natural logarithm, 2.718...)
	// we can just put appropriate if() and that equation, but itll take too much space. Ill just turn zeta =1
	//into zeta = 0.99999999999 or so. Itll do fine for our needs
	if (Zeta == 1.0) Zeta = 0.99999999999;
	if (mPLLSystem->DetectorType == Type_3 && Zeta < 1)
			PullOutRange = pi * OmegaN * exp(Zeta/sqrt(1 - pow(Zeta,2)) * atan(sqrt(1 - pow(Zeta,2))/Zeta));

//Due to the bug in 680x0 Math library code by metrowerks, atanh doesn't work (in CW10) PMac MathLib works fine though.
//I will define ATANH to be atanh for PPC and write my own implementation of atanh to be used under 680x0 code.

#if __POWERPC__
	#define	ATANH	atanh
#endif
	if ((mPLLSystem->DetectorType == Type_3) && (Zeta > 1))
			PullOutRange = pi * OmegaN * exp(Zeta/sqrt(pow(Zeta,2) - 1) * ATANH(sqrt(pow(Zeta,2) - 1)/Zeta));
	
	if ((mPLLSystem->DetectorType == Type_4) && (Zeta < 1))
			PullOutRange = 2 * pi * OmegaN * exp(Zeta/sqrt(1 - pow(Zeta,2)) * atan(sqrt(1 - pow(Zeta,2))/Zeta));

	if ((mPLLSystem->DetectorType == Type_4) && (Zeta > 1))
			PullOutRange = 2 * pi * OmegaN * exp(Zeta/sqrt(pow(Zeta,2) - 1) * ATANH(sqrt(pow(Zeta,2) - 1)/Zeta));

//And put calculated values back into structure
	mPLLSystem->Cc = Cc;
	mPLLSystem->R2 = R2;
	mPLLSystem->C = C;
	mPLLSystem->OmegaN = OmegaN;
	mPLLSystem->VCORangeMax = VCORangeMax;
	mPLLSystem->VCORangeMin = VCORangeMin;
	mPLLSystem->NMax = NMax;
	mPLLSystem->NMin = NMin;
	mPLLSystem->Bandwidth3db = Bandwidth3db;
	mPLLSystem->MaxOvershoot = MaxOvershoot;
	mPLLSystem->LockRange = LockRange;
	mPLLSystem->PullInRange = PullInRange;
	mPLLSystem->HoldRange = HoldRange;
	mPLLSystem->PullOutRange = PullOutRange;
	
return NO_ERRORS;
}

//----------------------------------------------------------------------------------
//		ShowTheResults()
// ---------------------------------------------------------------------------------
Int16	CPPMacPLLApp::ShowTheResults(void)
{
LCaption* theCaption;

	theCaption = (LCaption*)mWindow->FindPaneByID(kR1);
	theCaption->SetDescriptor(LStr255(mPLLSystem->R1/1000., FIXEDDECIMAL, 3));
	theCaption->Refresh();
	
	theCaption = (LCaption*)mWindow->FindPaneByID(kDividerNMax);
	theCaption->SetDescriptor(LStr255(mPLLSystem->NMax, FIXEDDECIMAL, 1));
	theCaption->Refresh();
	
	theCaption = (LCaption*)mWindow->FindPaneByID(kDividerNMin);
	theCaption->SetDescriptor(LStr255(mPLLSystem->NMin, FIXEDDECIMAL, 1));
	theCaption->Refresh();
	
	theCaption = (LCaption*)mWindow->FindPaneByID(kC);
	theCaption->SetDescriptor(LStr255(mPLLSystem->C/1.e-6, FIXEDDECIMAL, 6));
	theCaption->Refresh();
	
	theCaption = (LCaption*)mWindow->FindPaneByID(kCc);
	theCaption->SetDescriptor(LStr255(mPLLSystem->Cc/1e-6, FIXEDDECIMAL, 6));
	theCaption->Refresh();
	
	theCaption = (LCaption*)mWindow->FindPaneByID(kZetaMax);
	theCaption->SetDescriptor(LStr255(mPLLSystem->ZetaMax, FIXEDDECIMAL, 3));
	theCaption->Refresh();

	theCaption = (LCaption*)mWindow->FindPaneByID(kOmegaN);
	theCaption->SetDescriptor(LStr255(mPLLSystem->OmegaN/(2 * pi), FIXEDDECIMAL, 1));
	theCaption->Refresh();
	
	theCaption = (LCaption*)mWindow->FindPaneByID(kR2);
	theCaption->SetDescriptor(LStr255(mPLLSystem->R2/1000., FIXEDDECIMAL, 3));
	theCaption->Refresh();

	theCaption = (LCaption*)mWindow->FindPaneByID(kVCORangeMax);
	theCaption->SetDescriptor(LStr255(mPLLSystem->VCORangeMax/(2 * pi * 1.e6), FLOATDECIMAL, 3));
	theCaption->Refresh();

	theCaption = (LCaption*)mWindow->FindPaneByID(kVCORangeMin);
	theCaption->SetDescriptor(LStr255(mPLLSystem->VCORangeMin/(2 * pi * 1.e6), FLOATDECIMAL, 3));
	theCaption->Refresh();

	theCaption = (LCaption*)mWindow->FindPaneByID(kBandwidth3db);
	theCaption->SetDescriptor(LStr255(mPLLSystem->Bandwidth3db/(2 * pi), FIXEDDECIMAL, 1));
	theCaption->Refresh();

	theCaption = (LCaption*)mWindow->FindPaneByID(kMaxOvershoot);
	theCaption->SetDescriptor(LStr255(mPLLSystem->MaxOvershoot * 100.0, FIXEDDECIMAL, 2));
	theCaption->Refresh();

	theCaption = (LCaption*)mWindow->FindPaneByID(kLockRange);
	theCaption->SetDescriptor(LStr255(mPLLSystem->LockRange/(2 * pi), FIXEDDECIMAL, 1));
	theCaption->Refresh();

	theCaption = (LCaption*)mWindow->FindPaneByID(kPullInRange);
	theCaption->SetDescriptor(LStr255(mPLLSystem->PullInRange/(2 * pi), FIXEDDECIMAL, 1));
	theCaption->Refresh();

	theCaption = (LCaption*)mWindow->FindPaneByID(kHoldRange);
	theCaption->SetDescriptor(LStr255(mPLLSystem->HoldRange/(2 * pi), FIXEDDECIMAL, 1));
	theCaption->Refresh();

	theCaption = (LCaption*)mWindow->FindPaneByID(kPullOutRange);
	theCaption->SetDescriptor(LStr255(mPLLSystem->PullOutRange/(2 * pi), FIXEDDECIMAL, 1));
	theCaption->Refresh();

return NO_ERRORS;
}

//----------------------------------------------------------------------------------
//		CalculateTransientResponce()
// ---------------------------------------------------------------------------------
double_t	CPPMacPLLApp::CalculateTransientResponce(double_t zeta)
{

double_t	t;
//double_t	zeta = (mPLLSystem->ZetaMax + mPLLSystem->ZetaMin)/2;// dumping factor
double_t	omegan=1.0; //natural frequency
double_t	thetaO=mPLLSystem->Overshoot;

//I am taking inverce Laplace transform from He(s)=(s^2)/(s^2+2*zeta*omegan*s+omegan^2)
//(thats error-transfer function for secon-order system with active loop filter or
// with high loop gain (Ko*Kd >> omegan)
//The results are
//
//if zeta less then 1.0
//thetaO = (cos(sqrt(1-pow(zeta,2))*omegan*t)-((zeta/sqrt(1-pow(zeta,2)))*
//							sin(sqrt(1-pow(zeta,2))*omegan*t)))*exp(-zeta*omegan*t);
//if zeta == 1.0
//thetaO=(1-omegan*t)*exp(-omegan*t);
//
//and for zeta large then 1.0
//thetaO = (cosh(sqrt(pow(zeta,2)-1)*omegan*t)-((zeta/sqrt(pow(zeta,2)-1))
//					*sinh(sqrt(pow(zeta,2)-1)*omegan*t)))*exp(-zeta*omegan*t);
//
//
//this equations only applies for active loop filters or high loop gain systems with 
//passive filters. Equations for low loop gain system with pasive filters is slightly 
//different, and explanation for it could be found in any good PLL book. Equations for
// system with different order are NOT used in this program.
//
//Now that we know how to find thetaO given zeta, time and omegan, we need to
//find omegan given all other parameters.
//Since I dont know how to do that symbolically, Ill iterate untill omegan found.
//the other problem is to make sure that when system is underdumped, we do not find 
//local minimum in dumped oscillation, instead of getting acuall one. So, once we 
//found given oversoot, we have to still keep looking to make sure theta is not 
//going to get large.
// The algorithm is as follows: find time for which thetaO that satisfies given overshoot,
//for given zeta, if omegan = 1. That way, we can say that we found omegan*t (in radians)
//for such point. Then omegan could be found by dividing time found by transient responce time.
// of course, we want omegan be as large as possible, so transient responce should be 
//selected as short as possible for given application/overshoot.
//


//first, to find omegan*t:
// allocate array of 1000 points to hold thetaO for given zeta. We are going to keep 50
// radians worth of thetaO in the array, with 0.1 rad resolution. If system does not stabilizes
// over 50 rads `8 periods) wed probably dont want it. We also going to use abs(thetaO)
//for comparison with overshoot, since undershoot is important also (even though I call it 
//overshoot, its in fact a phase error that we want to avoid, and it can be above or below
//final value. But we will not put abs(theta) into our array, since itll make it much harder
// to find max. overshoot. The abs() will go into temp array.

ThrowIfMemError_ ();
ThrowIfMemFail_ (thetaArray);
//fill it in:

Int32	i;	//index for array

//get nonchanging parameters out of the loop to speed up calculations
double_t	zetasquare = pow(zeta,2);
double_t	sqrtoneminuszetasqr = sqrt(1-zetasquare);
double_t	sqrtzetasqrminusone = sqrt(zetasquare-1);
double_t	zetaomega = -zeta*omegan;
double_t	zetatosqrtoneminuszetasqr = zeta/sqrtoneminuszetasqr;
double_t	zetatosqrtzetasqrminusone = zeta/sqrtzetasqrminusone;
double_t	omegatosqrtoneminuszetasqr = sqrtoneminuszetasqr*omegan;
double_t	omegatosqrtzetasqrminusone = sqrtzetasqrminusone*omegan;

/*if (mPLLSystem->FilterType == Type_3 || mPLLSystem->FilterType == Type_4)
	for (i=0; i<THETA_ARRAY_SIZE; i++)
	{
		t = 0.1*i;
		if (zeta < 1.0) thetaArray[i] = (cos(omegatosqrtoneminuszetasqr*t)-
											(zetatosqrtoneminuszetasqr*
											sin(omegatosqrtoneminuszetasqr*t)))*
											exp(zetaomega*t);

		if (zeta == 1.0) thetaArray[i] = (1-omegan*t)*exp(-omegan*t);
	
		if (zeta > 1.0) thetaArray[i] = (cosh(omegatosqrtzetasqrminusone*t)-
											(zetatosqrtzetasqrminusone*
											sinh(omegatosqrtzetasqrminusone*t)))*
											exp(zetaomega*t);
		
	}*/
	
	if (mPLLSystem->FilterType == Type_3 || mPLLSystem->FilterType == Type_4){
		if (zeta < 1.0)
			for (i=0; i<THETA_ARRAY_SIZE; i++){
				t = 0.1*i;
				thetaArray[i] = (cos(omegatosqrtoneminuszetasqr*t)-
											(zetatosqrtoneminuszetasqr*
											sin(omegatosqrtoneminuszetasqr*t)))*
											exp(zetaomega*t);
			}
		if (zeta == 1.0)
			for (i=0; i<THETA_ARRAY_SIZE; i++){
				t = 0.1*i;
				thetaArray[i] = (1-omegan*t)*exp(-omegan*t);
			}
		if (zeta > 1.0)
			for (i=0; i<THETA_ARRAY_SIZE; i++){
				t = 0.1*i;
				thetaArray[i] = (cosh(omegatosqrtzetasqrminusone*t)-
											(zetatosqrtzetasqrminusone*
											sinh(omegatosqrtzetasqrminusone*t)))*
											exp(zetaomega*t);
			}
	}
	

double_t	VCOGain = mPLLSystem->VCOGain;
double_t	PhaseDetectorGain = mPLLSystem->PhaseDetectorGain;

//ANother coefficients that I took out of the loop to speed up calculations
double_t	coeff = (omegan/VCOGain*PhaseDetectorGain)-zeta;
double_t	coeff2 = (omegan/VCOGain*PhaseDetectorGain)-1;
double_t	coeff3 = coeff/sqrtoneminuszetasqr;

/*if (mPLLSystem->FilterType == Type_1 || mPLLSystem->FilterType == Type_2)
	for (i=0; i<THETA_ARRAY_SIZE; i++)
	{
		t = 0.1*i;
		if (zeta < 1.0) thetaArray[i] = (cos(omegatosqrtoneminuszetasqr*t)+
											coeff3*sin(omegatosqrtoneminuszetasqr*t))*
											exp(zetaomega*t);

		if (zeta == 1.0) thetaArray[i] = (1+(coeff2*omegan*t))*exp(-omegan*t);
	
		if (zeta > 1.0) thetaArray[i] = (cosh(omegatosqrtzetasqrminusone*t)+
											(coeff/sqrtzetasqrminusone)*
											sinh(omegatosqrtzetasqrminusone*t))*
											exp(zetaomega*t);
		
	}*/

	if (mPLLSystem->FilterType == Type_1 || mPLLSystem->FilterType == Type_2){
		if (zeta < 1.0)
			for (i=0; i<THETA_ARRAY_SIZE; i++){
				t = 0.1*i;
				thetaArray[i] = (cos(omegatosqrtoneminuszetasqr*t)+
											coeff3*sin(omegatosqrtoneminuszetasqr*t))*
											exp(zetaomega*t);
			}
		if (zeta == 1.0)
			for (i=0; i<THETA_ARRAY_SIZE; i++){
				t = 0.1*i;
				thetaArray[i] = (1+(coeff2*omegan*t))*exp(-omegan*t);
			}
		if (zeta > 1.0)
			for (i=0; i<THETA_ARRAY_SIZE; i++){
				t = 0.1*i;
				thetaArray[i] = (cosh(omegatosqrtzetasqrminusone*t)+
											(coeff/sqrtzetasqrminusone)*
											sinh(omegatosqrtzetasqrminusone*t))*
											exp(zetaomega*t);
			}		
	}


//Now that we have our array, we need to sort it to find omegan*t that satisfies error 
//requirements. Thats gonna be a little hairy. This is what Im going to do:
// Find all points that are near given overshoot, and then select latest one. The 
//only problem is to find nearest points. Since its a digital calculation, we most likely  
//will not have thetaO equial exactly the overshoot. Itll be somewhere near. The distance 
//(delta) will be different for given sampling rate depending on where on the curve the 
//particular point is. While there are many solutions for this problem, Ill use the simple 
//one, which while not optimal, will do the trick - check if given errors fits whithin 2 
//consecutive points, and if it is, record the time. The last time recorded will be either
//correct time, or, if system oscillating, isnt. Well have to check for that. If any point
//is large then error after found time, it oscillates.


Int32		index = -1;
double_t	omegant = -1; //If for some reasons not found, well know it
double_t	*tempTheta = new double_t[THETA_ARRAY_SIZE];
ThrowIfMemError_ ();
ThrowIfMemFail_ (tempTheta);

//copy thetaArray into tempTheta, taking abs of theta array in a process

	for (i = 0; i < THETA_ARRAY_SIZE; i++)
		tempTheta[i] = fabs(thetaArray[i]);

// now, do our thing	
	for (i = 0; i < THETA_ARRAY_SIZE-1; i++)
		{
			if  (((tempTheta[i]<= thetaO) && (thetaO <= tempTheta[i+1])) ||
				((tempTheta[i]>= thetaO) && (thetaO >= tempTheta[i+1])))
				{
					index = i+1;
					omegant=index * 0.1;
				}
		}
//And check to see if there are any problems happend during calculation
	for (i = index; i < THETA_ARRAY_SIZE; i++)
		if (tempTheta[i] >= thetaO)   Throw_(err_CalculationProblem); //return PROBLEM;
		
	
	if ((omegant == -1) || (index == -1)) Throw_(err_CalculationProblem); //return PROBLEM;
	
delete tempTheta;
return omegant;
}

//----------------------------------------------------------------------------------
//		FindMaxOvershoot()
// ---------------------------------------------------------------------------------
double_t	CPPMacPLLApp::FindMaxOvershoot(void)
{
double_t	Overshoot = 0;
Int32		i;

//we use thetaArray, already calculated, to see whats max overshoot responce is.
//We just need to find max value of thetaArray.
	for (i = 0; i < THETA_ARRAY_SIZE; i++)
		if (Overshoot < -thetaArray[i]) Overshoot = -thetaArray[i];


return Overshoot;
}

//----------------------------------------------------------------------------------
//		HideShowErrors()
// ---------------------------------------------------------------------------------
Int16	CPPMacPLLApp::HideShowErrors(PaneIDT whichError, Int16 SetClear)
{
	LCaption* theCaption;
	
	if (SetClear == SetError)
	{
		theCaption = (LCaption*)mWindow->FindPaneByID(whichError);
		theCaption->Show();
		theCaption->Refresh();
		SysBeep(3);
	}

	if (SetClear == ClearError)
	{
		theCaption = (LCaption*)mWindow->FindPaneByID(whichError);
		theCaption->Hide();
		theCaption->Refresh();
	}

return NO_ERRORS;
}

//----------------------------------------------------------------------------------
//		GetIputPopUps()
// ---------------------------------------------------------------------------------
Int16	CPPMacPLLApp::GetInputPopUps(void)

{			
	LStdPopupMenu			*thePopup;
	Int16					theDetectorType;
	Int16					theFilterType;
	Str255					theMenuItemName;

			// Get the popup menu.
			thePopup = (LStdPopupMenu *)mWindow->FindPaneByID( kPhaseDetectorPopUp);

			// Get the name of the menu to display.
			::GetMenuItemText(thePopup->GetMacMenuH(),thePopup->GetValue(),theMenuItemName);

			//Convert it to numerical representation
			theDetectorType = FindDetectorType(theMenuItemName);
			mPLLSystem->DetectorType = theDetectorType;
			
			// Get the popup menu.
			thePopup = (LStdPopupMenu *)mWindow->FindPaneByID( kLoopFilterPopUp);

			// Get the name of the menu to display.
			::GetMenuItemText(thePopup->GetMacMenuH(),thePopup->GetValue(),theMenuItemName);
			
			//Convert it to numerical representation
			theFilterType = FindLoopFilterType(theMenuItemName);
			mPLLSystem->FilterType = theFilterType;
			
	return NO_ERRORS;

}

//----------------------------------------------------------------------------------
//		ATANH()
// ---------------------------------------------------------------------------------
//Here is my own atanh() used in 680x0 code
double_t	ATANH(double_t x)
{
return log((1+x)/(1-x))/2;
}
