// ===========================================================================
//	MacPLL.h		1996 Mikhail Fridberg. All rights reserved.
// ===========================================================================

#pragma once

#include <LApplication.h>

//Structure to hold params of our PLL for easy access
typedef	struct
{
double_t	RefGeneratorFrequency;
Int32		ReferenceDivider;
double_t	PhaseDetectorGain;
Int32		NMin;
Int32		NMax;
double_t	VCOGain;
double_t	LockUpTime;
double_t	ZetaMin;
double_t	ZetaMax;
double_t	Overshoot;
double_t	R1;
double_t	R2;
double_t	C;
double_t	Cc;
double_t	OUTFreqMin;
double_t	OUTFreqMax;
double_t	VCORangeMax;
double_t	VCORangeMin;
double_t	OmegaN;
double_t	Bandwidth3db;
double_t	MaxOvershoot;
double_t	LockRange;
double_t	PullInRange;
double_t	HoldRange;
double_t	PullOutRange;
Int16		FilterType;
Int16		DetectorType;
} PLL_Params;

class	CPPMacPLLApp : public LApplication, public LListener, public LStr255 {
public:
						CPPMacPLLApp();		// constructor registers all PPobs
	virtual 			~ CPPMacPLLApp();		// stub destructor
	
		// this overriding function performs application functions
		
	virtual Boolean		ObeyCommand(CommandT inCommand, void* ioParam);	
	
		// this overriding function returns the status of menu items
		
	virtual void		FindCommandStatus(CommandT inCommand,
							Boolean &outEnabled, Boolean &outUsesMark,
							Char16 &outMark, Str255 outName);
							
	void				ListenToMessage( MessageT inMessage, void *ioParam );
	
	Int16				FindDetectorType(Str255 mDetectorType);
	Int16				FindLoopFilterType(Str255 mFilterType);
	Int16				CheckInputDataIntegrity(void);
	Int16				CheckOutputDataIntegrity(void);
	Int16				CalculateLoopFilter(void);
	Int16				ShowTheResults(void);
	Int16				HideShowErrors(PaneIDT whichError, Int16 SetClear);
	Int16				GetInputPopUps(void);
	double_t			CalculateTransientResponce(double_t zeta);
	double_t 			FindMaxOvershoot(void);
	
protected:
	LWindow *			MakeControlsWindow();
	virtual void		StartUp();		// overriding startup functions

private:
	LWindow *			mWindow; //holds main window
	PLL_Params			*mPLLSystem; //holds structure with entered and calculated PLL data
	double_t			*thetaArray; // hold array with transient response data

};

#define	INF	1.0/0.0
// declaration for my own implementation of atanh() used in 680x0 code
double_t				ATANH(double_t x);
